@@ -1,7 +1,9 @@
 use anyhow::{Error, Result};
 use env_logger::Env;
+use flate2::write::GzDecoder;
 use gzp::deflate::Gzip;
-use gzp::parz::{Compression, ParZ};
+use gzp::parz::Compression;
+use gzp::ZBuilder;
 use log::info;
 use std::fs::File;
 use std::io::{self, BufReader, BufWriter, Read, Write};
@@ -68,9 +70,13 @@ struct Opts {
     #[structopt(short, long, default_value = "3")]
     compression_level: u32,
 
-    // Number of compression threads to use, uses all available if not set
+    /// Number of compression threads to use, uses all available if not set
     #[structopt(short = "p", long)]
     compression_threads: Option<usize>,
+
+    /// Flag to switch to decompressing inputs. Note: this flag may change in future releases
+    #[structopt(short, long)]
+    decompress: bool,
 }
 
 fn main() -> Result<()> {
@@ -79,22 +85,36 @@ fn main() -> Result<()> {
         return Err(Error::msg("Invalid compression level"));
     }
 
-    if let Err(err) = run(
-        get_input(opts.file)?,
-        get_output(opts.output)?,
-        opts.compression_level,
-        opts.compression_threads.unwrap_or_else(num_cpus::get),
-    ) {
-        if is_broken_pipe(&err) {
-            exit(0)
+    if opts.decompress {
+        if let Err(err) = run_decompress(get_input(opts.file)?, get_output(opts.output)?) {
+            if is_broken_pipe(&err) {
+                exit(0)
+            }
+            return Err(err);
+        }
+    } else {
+        if let Err(err) = run_compress(
+            get_input(opts.file)?,
+            get_output(opts.output)?,
+            opts.compression_level,
+            opts.compression_threads.unwrap_or_else(num_cpus::get),
+        ) {
+            if is_broken_pipe(&err) {
+                exit(0)
+            }
+            return Err(err);
         }
-        return Err(err);
     }
     Ok(())
 }
 
-/// Run the program, returning any found errors
-fn run<R, W>(mut input: R, output: W, compression_level: u32, num_threads: usize) -> Result<()>
+/// Run the compression program, returning any found errors
+fn run_compress<R, W>(
+    mut input: R,
+    output: W,
+    compression_level: u32,
+    num_threads: usize,
+) -> Result<()>
 where
     R: Read,
     W: Write + Send + 'static,
@@ -103,13 +123,26 @@ where
         "Compressing with {} threads at compression level {}.",
         num_threads, compression_level
     );
-    // TODO: handle single threaded with write::GzEncoder
-    let mut parz: ParZ<Gzip> = ParZ::builder(output)
+    let mut writer = ZBuilder::<Gzip, _>::new()
+        .num_threads(num_threads)
         .compression_level(Compression::new(compression_level))
-        .num_threads(num_threads)?
-        .build();
-    io::copy(&mut input, &mut parz)?;
-    parz.finish()?;
+        .from_writer(output);
+    io::copy(&mut input, &mut writer)?;
+    writer.finish()?;
+    Ok(())
+}
+
+/// Run the compression program, returning any found errors
+fn run_decompress<R, W>(mut input: R, output: W) -> Result<()>
+where
+    R: Read,
+    W: Write + Send + 'static,
+{
+    info!("Decompressing.");
+
+    let mut writer = GzDecoder::new(output);
+    io::copy(&mut input, &mut writer)?;
+    writer.finish()?;
     Ok(())
 }
 /// Parse args and set up logging / tracing
